/*************************************************
 **                                             **
 **   dbf.h                                     **
 **                                             **
 **          Copyright (C) 1988-1992 by         **
 **               SoftC, Ltd.                   **
 **               16820 3rd St NE               **
 **               Anoka, MN 55304               **
 **               (612) 434-6968                **
 **                                             **
 **             All rights reserved.            **
 *************************************************/



#ifndef __SOFTC_H__
#include "softc.h"
#endif



/**********
 *
 *  Type Definitions
 *
 **********/




typedef struct {  /* database field description */
  CHAR name[12];    /* field name */
  INT type;         /* field type
		     * 'c' or 'C' - character
		     * 'd' or 'D' - date
		     * 'f' or 'F' - floating point (binary)
		     * 'l' or 'L' - logical
		     *              't' or 'T' - true
		     *              'f' or 'F' - false
		     *              'y' or 'Y' - yes
		     *              'n' or 'N' - no
		     *              '?'        - uninitialized
		     * 'n' or 'N' - floating point (bcd) */
  INT len;          /* field width */
  INT decpl;        /* decimal count */
} SC_FIELD;


/* SoftC extended data file field types */
#define SC_SHORT_FLD 1              /* signed short integer (16 bits) */
#define SC_USHORT_FLD 2             /* unsigned short integer */
#define SC_LONG_FLD 3               /* signed long integer (32 bits) */
#define SC_ULONG_FLD 4              /* unsigned long integer */
#define SC_FLOAT_FLD 11             /* IEEE single real (32 bits) */
#define SC_MSFLT_FLD 12             /* Microsoft floating point (old) */
#define SC_DOUBLE_FLD 13            /* IEEE double real (64 bits) */
#define SC_MSDBL_FLD 14             /* Microsoft double (old) */



typedef struct {  /* dBASE data file information packet */
  ULONG ladrs;          /* beginning address of locked area */
  ULONG lsize;          /* length of locked region */
  SC_FLAGS flags;       /* miscellaneous flags */
  CHAR fname[80];       /* file name */
  INT style;            /* file type (dBase3 or dBase4) */
  INT memo;             /* memo file required */
  INT mdx;              /* dBase4 .MDX file required (true or false) */
  INT trans;            /* dBase4 transaction in process (true or false) */
  INT encrypt;          /* dBase4 data file encrypted (true or false) */
  INT lockt;            /* record/file/no lock */
} SC_DBFINFO;

/* SC_DBFINFO memo flag */
#define SC_USEMEMO 1                /* memo file used */
#define SC_NOMEMO 0                 /* no memo file required */

typedef struct {  /* dBASE data file record information packet */
  INT reclen;     /* record length in bytes */
  INT numflds;    /* number of fields in record */
  CHAR F_PTR bfr; /* address of the record buffer */
} SC_DBFRINFO;


#ifndef _Windows
#define scddfname(dbf,field)   (((SC_DBFFILE *)(sc_base.p[dbf].p.a))->f.a[field].n)
#define scddftype(dbf,field)   (((SC_DBFFILE *)(sc_base.p[dbf].p.a))->f.a[field].t)
#define scddflength(dbf,field) (((SC_DBFFILE *)(sc_base.p[dbf].p.a))->f.a[field].l)
#define scddfdecpl(dbf,field)  (((SC_DBFFILE *)(sc_base.p[dbf].p.a))->f.a[field].d)
#define scddlastrec            (sc_df->h.n)
#endif


#ifdef __ANSI_PROTOTYPES__
#ifdef __cplusplus
extern "C" {
#endif


/**********
 *
 *  dBASE Database File Manipulator Prototypes
 *
 **********/


INT PROTOS scddbfrsz(  INT,              /* data file handle */
		       INT F_PTR,        /* I/O buffer length in records */
		       INT );            /* set or get buffer length */

INT PROTOS scddbof(    INT );            /* file handle */

INT PROTOS scddclose(  INT );            /* file handle */

INT PROTOS scddcreate( CHAR F_PTR,       /* filename */
		       INT,              /* number of data fields */
		       SC_FIELD F_PTR,   /* field description array */
		       INT );            /* dBase version (3 or 4) */

INT PROTOS scddeof(    INT );            /* file handle */

INT PROTOS scddflush(  INT );            /* file handle */

INT PROTOS scddhget(   INT );            /* file handle */

INT PROTOS scddhlock(  INT );            /* file handle */

INT PROTOS scddinfo(   INT,              /* file handle */
		       SC_DBFINFO F_PTR ); /* file information & status */

INT PROTOS scddlock(   INT );            /* file handle */

INT PROTOS scddlud(    INT,              /* file handle */
		       CHAR F_PTR,       /* last update date string */
		       INT );            /* date string format */

INT PROTOS scddopenx(  INT F_PTR,        /* file handle */
		       CHAR F_PTR,       /* filename */
		       INT );            /* open mode & characteristics */

INT PROTOS scddpack(   INT F_PTR);       /* data file handle */

INT PROTOS scddsize(   INT,              /* file handle */
		       LONG F_PTR );     /* number of data records */

INT PROTOS scddunlock( INT );            /* file handle */




/**********
 *
 *  dBASE Database Field Manipulator Prototypes
 *
 **********/




INT PROTOS scddfget(   INT,              /* file handle */
		       INT,              /* field number */
		       VOID F_PTR );     /* buffer for field data */

INT PROTOS scddfgets(  INT,              /* file handle */
		       INT,              /* field number */
		       CHAR F_PTR );     /* buffer for field data */

INT PROTOS scddfinfo(  INT,              /* file handle */
		       INT F_PTR,        /* length of longest field */
		       SC_FIELD F_PTR);  /* field description array */

INT PROTOS scddfnam2no(INT,              /* file handle */
		       INT F_PTR,        /* field number */
		       CHAR F_PTR);      /* field name */

INT PROTOS scddfnget(  INT,              /* file handle */
		       CHAR F_PTR,       /* field name */
		       VOID F_PTR );     /* buffer for field data */

INT PROTOS scddfngets( INT,              /* file handle */
		       CHAR F_PTR,       /* field name */
		       CHAR F_PTR );     /* buffer for field data */

INT PROTOS scddfno2nam(INT,              /* file handle */
		       CHAR F_PTR,       /* field name */
		       INT );            /* field number */

INT PROTOS scddfnput(  INT,              /* file handle */
		       CHAR F_PTR,       /* field name */
		       VOID F_PTR );     /* buffer for field data */

INT PROTOS scddfnputs( INT,              /* file handle */
		       CHAR F_PTR,       /* field name */
		       CHAR F_PTR );     /* buffer for field data */

INT PROTOS scddfput(   INT,              /* file handle */
		       INT,              /* field number */
		       VOID F_PTR );     /* buffer for field data */

INT PROTOS scddfputs(  INT,              /* file handle */
		       INT,              /* field number */
		       CHAR F_PTR );     /* buffer for field data */



/**********
 *
 *  dBASE Database Record Manipulator Prototypes
 *
 **********/




INT PROTOS scddrclear( INT );            /* file handle */

INT PROTOS scddrdel(   INT,              /* file handle */
		       LONG );           /* record number to delete */

INT PROTOS scddrget(   INT,              /* file handle */
		       LONG );           /* record number to read */

INT PROTOS scddrgetx(  INT,              /* file handle */
		       CHAR F_PTR,       /* input buffer address */
		       LONG );           /* record number to read */

INT PROTOS scddrinfo(  INT,              /* file handle */
		       SC_DBFRINFO F_PTR ); /* record info structure */

INT PROTOS scddrlock(  INT,              /* file handle */
		       LONG );           /* record number to lock */

INT PROTOS scddrnum(   INT,              /* file handle */
		       LONG F_PTR );     /* current data record number */

INT PROTOS scddrput(   INT,              /* file handle */
		       LONG F_PTR,       /* record number */
		       INT );            /* append or update record */

INT PROTOS scddrputx(  INT,              /* file handle */
		       CHAR F_PTR,       /* output buffer address */
		       LONG F_PTR,       /* record number */
		       INT );            /* append or update record */

INT PROTOS scddrstat(  INT );            /* file handle */

INT PROTOS scddrundel( INT,              /* file handle */
		       LONG );           /* record number */


#ifdef __cplusplus
}
#endif
#endif




